function plugindef()
   finaleplugin.RequireSelection = true
   finaleplugin.Author = "Harald Schollmeyer"
   finaleplugin.Version = "1.0"
   finaleplugin.Date = "12.4.2019"
   return "CC64 to Pedal Expressions", "CC64 to Pedal Expressions", "Inserts pedal expressions according to MIDI CC64 events"
end


-- global helper variable
str = finale.FCString()

-- Function to format controls
function FormatCtrl( ct, h, w, st)
    ct:SetHeight(h)
    ct:SetWidth(w)
    str.LuaString = st
    ct:SetText(str)
end

function SearchHSPedalMarkings()
local x,y = 0,0
local allexprdefs = finale.FCTextExpressionDefs()
allexprdefs:LoadAll()
local count=0
for exprdef in each(allexprdefs) do
    allexprdefs:GetItemAt(count):GetDescription(str)
    count = count + 1
    if (str.LuaString == "HSPedalMarkingPedalPress") then x = count end
    if (str.LuaString == "HSPedalMarkingPedalRelease") then y= count end
end
return x,y
end

-- Function to create new Pedal Expressions in the document's expression library
-- Thanks to Bart Visser on the JWLua mailing list Wed Oct 21 21:30:28 CEST 2015
function CreateExpression(expressionTextAsLuaString, descriptionAsLuaString, controllernumber, value)
	
	local expressionString = finale.FCString()
	expressionString.LuaString = "^fontMus(Font0,0)^size(24)^nfx(0)"..expressionTextAsLuaString
	
	local descriptionStr = finale.FCString()
	descriptionStr.LuaString = descriptionAsLuaString
	
	local newExpression = finale.FCTextExpressionDef()
	newExpression.UseCategoryFont = true
	newExpression:SaveNew()
	newExpression:SaveNewTextBlock(expressionString)
	newExpression.CategoryID = finale.DEFAULTCATID_EXPRESSIVETEXT 
	newExpression.UseCategoryPos = false

	newExpression.HorizontalAlignmentPoint = finale.ALIGNHORIZ_CENTERPRIMARYNOTEHEAD 
	newExpression.HorizontalOffset = 0
    newExpression.HorizontalJustification = finale.EXPRJUSTIFY_CENTER

	newExpression.VerticalAlignmentPoint = finale.ALIGNVERT_BELOW_STAFF_BASELINE_OR_ENTRY 	
	newExpression.VerticalBaselineOffset = -20
    newExpression.VerticalEntryOffset = -60

	newExpression:SetDescription(descriptionStr)
    newExpression:SetPlaybackController(controllernumber, value)
	newExpression:Save()
	return newExpression.ItemNo
end


-- Create the dialog and its controls
local dialog = finale.FCCustomLuaWindow()
str.LuaString = "CC64 to Pedal Expressions"
dialog:SetTitle(str)
dialog:SetClientWidth(300)
local static1 = dialog:CreateStatic(0, 0)
FormatCtrl( static1, 120, 300,
[[This plug-in script inserts expressions for pedal markings
where MIDI CC64 events are found.

Select a region in a piano system (=2 staves).
Pedal expressions will be added under the lower staff.

If you don't have your own pedal expressions, click here:
(expressions won't be created twice)]])

local static4 = dialog:CreateStatic(0, 140)
FormatCtrl( static4, 120, 300,"Otherwise select your own expressions for pedal use:")

local CreatePedalMarkingsButton = dialog:CreateButton(0,110)
FormatCtrl( CreatePedalMarkingsButton, 20, 280, "Create Pedal Press/Release Expressions")

local expPedalPress, expPedalRelease = SearchHSPedalMarkings()

local d = 160
local static2 = dialog:CreateStatic(40, d)
FormatCtrl( static2, 40, 200, "Select expression for 'pedal press':")
local edit1 = dialog:CreateEdit(40, d+20)
    edit1:SetInteger( expPedalPress )
local ExpressionButton1 = dialog:CreateButton(120,d+20)
FormatCtrl( ExpressionButton1, 30, 120, "Select Expression")

local static3 = dialog:CreateStatic(40, d+60)
FormatCtrl( static3, 40, 200, "Select expression for 'pedal release':")
local edit2 = dialog:CreateEdit(40, d+80)
    edit2:SetInteger( expPedalRelease )
local ExpressionButton2 = dialog:CreateButton(120,d+80)
FormatCtrl( ExpressionButton2, 30, 120, "Select Expression")

chk = dialog:CreateCheckbox(40,d+120)
FormatCtrl( chk, 20,200,"Delete original MIDI CC64 events")

dialog:CreateOkButton()
dialog:CreateCancelButton()


-- Callback function to handle the expression selection buttons
function callback(ctrl)
    local ui = finenv.UI()

    if (ctrl:GetControlID() == ExpressionButton1:GetControlID()) then
    local myIDResult = ui:DisplayTextExpressionDialog( edit1:GetInteger() )
    if (myIDResult > 0) then edit1:SetInteger( myIDResult ) end
    end

    if (ctrl:GetControlID() == ExpressionButton2:GetControlID()) then    
     local myIDResult = ui:DisplayTextExpressionDialog( edit2:GetInteger() )
    if (myIDResult > 0) then edit2:SetInteger( myIDResult ) end
    end

    -- Handle CreatePedalMarkingsButton 
    if (ctrl:GetControlID() == CreatePedalMarkingsButton:GetControlID()) then
        -- search HSPedalMarkings. If they don't exist, create them
        local b,v = SearchHSPedalMarkings()
        if (b == 0) and (v ==0 ) then
            edit1:SetInteger( CreateExpression("¡", "HSPedalMarkingPedalPress", 64, 127) )
            edit2:SetInteger( CreateExpression("*", "HSPedalMarkingPedalRelease", 64, 0) )
        end
    end

end
dialog:RegisterHandleCommand( callback)

-- MAIN programm start
-- execute the dialog if confirmed with OK
if ( dialog:ExecuteModal(nil) == finale.EXECMODAL_OK ) then

    local measures = finale.FCMeasures()
    measures:LoadRegion( finenv.Region() )

    for m in each( measures ) do

        -- FCMidiExpressions are cell-bound, so we have to go through all cells
        local cell = finale.FCCell( m.ItemNo, finenv.Region().StartStaff )
        local midiexpressions = finale.FCMidiExpressions()
        midiexpressions:LoadAllInCell( cell )

        -- "eachbackwards" is used to be able to delete from the collection
        for me in eachbackwards(midiexpressions) do

            -- check for CC value 64: >0 means pedal pressed, =0 means pedal released
            if (me.ControllerNumber == 64)  then
            if (me.ControllerValue > 0) then id = edit1:GetInteger() else id = edit2:GetInteger() end
                -- insert the appropriate expression
                local ex = finale.FCExpression()
                ex:SetID( id )
                ex:SetMeasurePos( me.MeasurePos ) 
                ex:SaveNewToCell( finale.FCCell( me.Measure, finenv.Region().EndStaff ) )
            end

            -- Delete original CC64s, if checkbox is checked
            if (chk:GetCheck() > 0) then me:DeleteData() end
        end

        cell:Save()
    end
end
